//ONLY WORKS WITH LATEST FIRMWARE

// Set the default resolution to 640 x 480
#define BLACKFIN_RESOLUTION_DEFAULT BLACKFIN_320_BY_240
#define BLACKFIN_X_RES 320
#define BLACKFIN_Y_RES 240


// Include library files
#include <Cameras/Surveyor/blackfin.h>

// Define the UART for the camera
BLACKFIN_CAMERA camera = MAKE_BLACKFIN_CAMERA(UART_BLACKFIN);


//store data in a struct
typedef struct s_CAMERDATA{
	uint16_t  bleft;
	uint16_t  bright;
	uint16_t  btop;
	uint16_t  bbottom;
	uint16_t  centerX;
	uint16_t  centerY;
	uint32_t numPixels;
	} CAMERDATA;
CAMERDATA cameraData[3];//number of informations to store


uint8_t numBlobs;

COLOR min;
COLOR max;



//set ERP to track red ball
//colors taken from console: http://www.webbot.org.uk/blackfin/blackfin.jnlp
void blackfin_find_red_ball(void)
	{
	//Assign the variables an RGB value such as
	//colorSetRGB(&min,0,0,0);
	//colorSetRGB(&max,250,250,250);
	//colorSetYUV(&min,0,48,85);
	//colorSetYUV(&max,210,215,220);

	//correct values
	colorSetYUV(&min,50,88,185);
	colorSetYUV(&max,110,115,220);

	//write colors to a bin number
	//The Lib will automatically convert to YUV for you eg for bin#0:
	blackfinSetBinRange(&camera,0, &min, &max);
	}

//my fridge is bright blue, find it!
//colors taken from console: http://www.webbot.org.uk/blackfin/blackfin.jnlp
void blackfin_find_blue_fridge(void)
	{
	//Assign the variables an RGB value such as
	colorSetRGB(&min,45,145,95);
	colorSetRGB(&max,140,165,120);

	//write colors to a bin number
	//The Lib will automatically convert to YUV for you eg for bin#0:
	blackfinSetBinRange(&camera,0, &min, &max);
	}

/*
this code worked for original Blackfin code for MOBOT:
Blackfin_Bin_Range(0, (Y_mean+80), 255, 0, 255, 0, 255);//works
*/

//looks at blob data, stores into usefull array
//pass blob number, and array to store data in
//blob numbers start at 0
void Blackfin_Store_Blob(uint8_t blobNo)
	{
	uint8_t numBlobs = blackfinDetectBlobs(&camera,0);

	const BLACKFIN_BLOB*  blob = blackfinFetchBlob(&camera, blobNo);
	cameraData[0].bleft=blob->left;
	cameraData[0].bright=blob->right;
	cameraData[0].btop=blob->top;
	cameraData[0].bbottom=blob->bottom;
	cameraData[0].centerX = blob->xCenter;
	cameraData[0].centerY = blob->yCenter;
	cameraData[0].numPixels = blob->pixels;

	//test output
	//rprintf("\n blob: x:%d y:%d px:%lu",cameraData[0].centerX,cameraData[0].centerY,cameraData[0].numPixels);
	}

//compares number of pixels to minimum desired blob size
//if pixel size is greater than desired, returns true
boolean Blackfin_Pixel_Check(uint32_t confidence)
	{
	//rprintf("\n conf:%lu pix:%lu",confidence,cameraData[0].numPixels);
	
	if (cameraData[0].numPixels>=confidence)//if pixels greater than confidence
		return 1;
	else//blob is too small
		return 0;
	}

//moves robot body to match head
//damping ratio is how fast to change, and 
void Blackfin_BodyTrack(uint8_t damping_ratio, int8_t error_X)
	{
	//find new cam locations
	//int16_t mod_wheel_left = act_getSpeed(&wheel_left)-error_X/damping_ratio;
	//int16_t mod_wheel_right = act_getSpeed(&wheel_right)+error_X/damping_ratio;
	int16_t mod_wheel_left = -error_X/damping_ratio;
	int16_t mod_wheel_right = error_X/damping_ratio;

	//make sure current position + error doesn't exceed camera motion bounds
	mod_wheel_left=CLAMP(mod_wheel_left, DRIVE_SPEED_MIN, DRIVE_SPEED_MAX);
	mod_wheel_right=CLAMP(mod_wheel_right, DRIVE_SPEED_MIN, DRIVE_SPEED_MAX);

	//current wheel speed modified by camera - doesn't yet prevent maxing out
	act_setSpeed(&wheel_left,mod_wheel_left);
	act_setSpeed(&wheel_right,mod_wheel_right);
	}

//moves camera to center on target: low numbers are top left, high are bottom right
//accepts: confidence (min # of pixels to track)
//higher damping ratio numbers reduce cam oscillation, but also slows it down
//body, 0 or 1 for body tracking
void Blackfin_HeadTrack(uint32_t confidence, uint8_t damping_ratio, uint8_t body)
	{
	//find max error of camera position compared to object
	//error=(center of object in cam - actual center)
	int16_t blackfin_position_error_X = (int16_t)(cameraData[0].centerX - BLACKFIN_X_RES/2)/damping_ratio;
	int16_t blackfin_position_error_Y = (int16_t)(cameraData[0].centerY - BLACKFIN_Y_RES/2)/damping_ratio;

	//find new cam locations
	int16_t new_cam_location_X = act_getSpeed(&camera_hor)-blackfin_position_error_X;
	int16_t new_cam_location_Y = act_getSpeed(&camera_ver)+blackfin_position_error_Y;

	//make sure current position + error doesn't exceed camera motion bounds
	new_cam_location_X=CLAMP(new_cam_location_X, -66, 66);
	new_cam_location_Y=CLAMP(new_cam_location_Y, DRIVE_SPEED_MIN, 34);

	//convert blackfin coordinates to camera servo coordinates
	//blackfin_position_error_X = interpolate(blackfin_position_error_X, -BLACKFIN_X_RES/2, BLACKFIN_X_RES/2, DRIVE_SPEED_MIN, DRIVE_SPEED_MAX);
	//blackfin_position_error_Y = interpolate(blackfin_position_error_Y, -BLACKFIN_Y_RES/2, BLACKFIN_Y_RES/2, DRIVE_SPEED_MIN, DRIVE_SPEED_MAX);


	if(Blackfin_Pixel_Check(confidence))
		{
		//move camera to new location
		act_setSpeed(&camera_hor,new_cam_location_X);
		act_setSpeed(&camera_ver,new_cam_location_Y);//fix location for MOBOT

		//output for testing
		rprintf("\n hor:%d ver:%d",new_cam_location_X,new_cam_location_Y);

		if(body==1)//body tracking on
			Blackfin_BodyTrack(2,new_cam_location_X);//10 is acceptable speed
		}
	}


/*
//to go through many blobs
// Iterate through each blob
for(uint8_t blobNo=0; blobNo < numBlobs; blobNo++)
	{
	const BLACKFIN_BLOB*  blob = blackfinFetchBlob(&camera, blobNo);

	cam_object1[]=blob->left;
	blob->right;
	blob->top;
	blob->bottom;
	blob->xCenter;
	blob->yCenter;
	blob->pixels;
  // The info on the returned blob is documented in 'blackfin.h' - look at the definition of the structure BLACKFIN_BLOB

 // eg the center of the blob is at:  blob->xCenter and blob->yCenter

 // or the 'size' of the blob can be read using blob->pixels

}
*/
