/****************************************************************************
*
*   Copyright (c) 2009 www.societyofrobots.com
*   (please link back if you use this code!)
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License version 2 as
*   published by the Free Software Foundation.
*
*   Alternatively, this software may be distributed under the terms of BSD
*   license.
*
****************************************************************************/

//download the manual and latest version of WebbotLib here
//http://webbotavrclib.sourceforge.net/
//adapted for societyofrobots.com

//define how data should be sent (choose only one)
//#define WIRELESS
#define USB

//define which program to run
//#define ANGLE_FINDER	//use this to find new joint angles
#define HEAD_TRACK		//head will track object
//#define PLAY_FETCH		//find ball, grab it, then go to blue fridge with it
//#define MAN_CONTROL		//keyboard controlled
//#define LINE_FOLLOW		//follow a line
//#define RANDOM_DANCE		//dance randomly


//allows for printing with features, but uses more memory
#define RPRINTF_FLOAT
#define RPRINTF_COMPLEX

//define buffer size for Blackfin (must be called here before other stuff)
#if defined(HEAD_TRACK) || defined(LINE_FOLLOW)
	#define UART3_RX_BUFFER_SIZE 100 //at least 80, but usually more
#endif


//additional math capabilities
#include <math.h>//for pi and trig
#include <stdlib.h>//to get random numbers
#include <avr/pgmspace.h>//for changing how memory is stored

//WebbotLib Includes
#include "buffer.h"
#include "sys/axon2.h"//declare microcontroller
#include "servos.h"		//use for servos
#include "a2d.h"		//use for ADC
#include "rprintf.h"	//use for UART
//#include "i2c_master.h"	//use for I2C
//#include "errors.h"		//helpful for debugging mistakes

//user variables
uint16_t speed_u=100;	//speed of robot motions
int16_t theta_u=0;		//direction of travel, from 100 to -100
int16_t turning_u=0;	//rate of rotation, from 100 to -100
int16_t head_adjust=0;	//user modified head location
int cByte=-1;			//store command data
const uint16_t stop=1000;//the value that represents stop
boolean shutdown=0;		//shut down all major systems (1 means shut down)


//user includes
#include "hardware.h"	//declare your servo and sensor ports here
#include "special_motions.h" //ERP motions that aren't entire gaits, just single thingies


//special program includes
#ifdef ANGLE_FINDER
	#include "kinematics.h"	//modified SoR Biped Engine v3.0
	#include "angle_finder.h" //manually find new joint angles
#endif

#ifdef HEAD_TRACK
	#include "ERP_Blackfin.h" //blackfin stuff for ERP
	#include "misc.h" //ERP will follow object with head
#endif

#ifdef PLAY_FETCH
	#include "kinematics.h"	//modified SoR Biped Engine v3.0
	#include "ERP_Blackfin.h" //blackfin stuff for ERP
	#include "misc.h" //ERP will follow object with head
#endif

#ifdef MAN_CONTROL
	#include "manual_control.h" //use keyboard to control robot
#endif

#ifdef LINE_FOLLOW
	#include "ERP_Blackfin.h" //blackfin stuff for ERP
	#include "head_track.h"
	#include "line_follow.h" //ERP will follow line
#endif

#ifdef RANDOM_DANCE
	#include "random_number_gen.h" //generates TRUE random number
	#include "random_dance.h" //randomly picks preprogrammed motions to dance to
#endif


//initialize hardware, ie servos, UART, etc.
void appInitHardware(void)
	{
	delay_ms(100);//start up delay so motors don't jump

	//servoPWMInit(&bank1);//initialize PWM for actuators
	servosInit(&bank2, TIMER3_COMPAREA);//software based PWM (use diff timer per bank, don't use PWM pin timers for it)

	turn_actuators_OFF();//keep actuators off until everything is initiated
	led_off();
		
	//setup UART on USB, too slow and it'll slow your entire bot down
	uartInit(U0_UART, BLUETOOTH_BAUD);//bluetooth
	uartInit(USB_UART, USB_BAUD);//230400 is as fast as I can get it
	uartInit(U2_UART, 115200);
	uartInit(UART_BLACKFIN, BLACKFIN_BAUD);//camera

	#ifdef RANDOM_DANCE
		init_rand_gen();//initialize random number generator
	#endif

	// Set which uart is default during startup
	#ifdef USB
		rprintfInit(USB_ACTIVATE);
	#endif
	#ifdef WIRELESS
		rprintfInit(WIRELESS_ACTIVATE);
	#endif

	
	//init sensors
	distanceInit(sharp_cam);
	#if defined(HEAD_TRACK) || defined(LINE_FOLLOW)
		blackfinInit(&camera);
	#endif

	update_sensors_all();

	rprintf("\nAxon II initiated.\n\n");
	
	
	//while(!button_pressed());//don't do anything until button is pushed

	turn_actuators_ON(); //activate servos

	//while(!button_pressed());//don't do anything until button is pushed

	//arm_fold();
	}

TICK_COUNT appInitSoftware(TICK_COUNT loopStart)
	{
	return 0;
	}

// This is the main loop
TICK_COUNT appControl(LOOP_COUNT loopCount, TICK_COUNT loopStart)
	{
	#ifdef ANGLE_FINDER
		angle_finder();
	#endif

	#ifdef HEAD_TRACK
		head_track();

		/*blackfin_find_red_ball();
		while(1)
			{
			//rprintf("test");
			Blackfin_Store_Blob(1);//blob numbers start at 1
			//head_track();
			Blackfin_HeadTrack(10,15);//confidence, damping ratio
			}*/
			
	#endif

	#ifdef PLAY_FETCH
		play_fetch();
	#endif

	#ifdef MAN_CONTROL
		manual_control();
	#endif

	#ifdef LINE_FOLLOW
		line_follow();
	#endif

	#ifdef RANDOM_DANCE
		random_dance();
	#endif



	////////////////////////////////////

	//tell actuators to move
	//biped_engine(theta_u, turning_u, speed_u);
	//biped_engine(0, 0, 200);//use for testing purposes


	return 2000; // 2ms delay to stop crazy oscillations (just make sure about ~20ms passes per loop)
	}
