/****************************************************************************
*
*   Copyright (c) 2009 www.societyofrobots.com
*   (please link back if you use this code!)
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License version 2 as
*   published by the Free Software Foundation.
*
*   Alternatively, this software may be distributed under the terms of BSD
*   license.
*
****************************************************************************/



//initialize servo location variables
//these variables store only current servo location
int16_t ALB=0;
int16_t ALJ1=0;
int16_t ALJ2=0;
int16_t ALC=0;
int16_t ARB=0;
int16_t ARJ1=0;
int16_t ARJ2=0;
int16_t ARC=0;
int16_t CAMh=0;
int16_t CAMv=0;

//gait percentages
int16_t gait_per_a;
int16_t gait_per_b;


//HARDCODE MAIN GAITS HERE, values must be from -100 to +100 only
uint8_t array_count=8;//number of positions in the gait arrays

//define forward gait (start at extreme, lift back leg to front, slowly sweep forward)
int16_t ALB_forward[8]={0,0,0,0,0,0,0,0};
int16_t ALJ1_forward[8]={0,0,0,0,0,0,0,0};
int16_t ALJ2_forward[8]={0,0,0,0,0,0,0,0};
int16_t ALC_forward[8]={0,0,0,0,0,0,0,0};
int16_t ARB_forward[8]={0,0,0,0,0,0,0,0};
int16_t ARJ1_forward[8]={0,0,0,0,0,0,0,0};
int16_t ARJ2_forward[8]={0,0,0,0,0,0,0,0};
int16_t ARC_forward[8]={0,0,0,0,0,0,0,0};
int16_t CAMh_forward[8]={0,0,0,0,0,0,0,0};
int16_t CAMv_forward[8]={0,0,0,0,0,0,0,0};

//define rotate gait
int16_t ALB_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ALJ1_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ALJ2_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ALC_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ARB_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ARJ1_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ARJ2_rotate[8]={0,0,0,0,0,0,0,0};
int16_t ARC_rotate[8]={0,0,0,0,0,0,0,0};
int16_t CAMh_rotate[8]={0,0,0,0,0,0,0,0};
int16_t CAMv_rotate[8]={0,0,0,0,0,0,0,0};

//define side-step
int16_t ALB_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ALJ1_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ALJ2_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ALC_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ARB_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ARJ1_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ARJ2_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t ARC_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t CAMh_sidestep[8]={0,0,0,0,0,0,0,0};
int16_t CAMv_sidestep[8]={0,0,0,0,0,0,0,0};

//define stand still gait
int16_t ALB_stand[8]={0,0,0,0,0,0,0,0};
int16_t ALJ1_stand[8]={0,0,0,0,0,0,0,0};
int16_t ALJ2_stand[8]={0,0,0,0,0,0,0,0};
int16_t ALC_stand[8]={0,0,0,0,0,0,0,0};
int16_t ARB_stand[8]={0,0,0,0,0,0,0,0};
int16_t ARJ1_stand[8]={0,0,0,0,0,0,0,0};
int16_t ARJ2_stand[8]={0,0,0,0,0,0,0,0};
int16_t ARC_stand[8]={0,0,0,0,0,0,0,0};
int16_t CAMh_stand[8]={0,0,0,0,0,0,0,0};
int16_t CAMv_stand[8]={0,0,0,0,0,0,0,0};


//various common initialized variables
uint8_t i=0;
uint16_t j=0;
uint16_t k=0;

//number of times to output data and/or read sensors per full fin cycle
uint16_t output_res=10;//too low, and the timer *might* overflow

//interpolation variables
float per_a;
float per_b;
uint8_t next=0;



//this function calculates gait meshing
void calculate_per(uint16_t speed, uint8_t gait_count)
	{
	//mesh current gait position with future gait position
	per_a = ((float)(speed-j)) / ((float)speed);
	per_b = (float)1.0-per_a;

	//make sure future gait position isn't outside array
	if(i!=gait_count-1)
		next=i+1;
	else //if it is, then restart back at zero
		next=0;
	}

//command robot to do stuff (servo name, servo position)
void robot_move(void)
	{
	act_setSpeed(&arm_left_base, ALB);
	act_setSpeed(&arm_left_J1, ALJ1);
	act_setSpeed(&arm_left_J2, ALJ2);
	act_setSpeed(&arm_left_claw, ALC);
	act_setSpeed(&arm_right_base, ARB);
	act_setSpeed(&arm_right_J1, ARJ1);
	act_setSpeed(&arm_right_J2, ARJ2);
	act_setSpeed(&arm_right_claw, ARC);
	act_setSpeed(&camera_hor, CAMh);
	act_setSpeed(&camera_ver, CAMv);
	}


//set all servo values to zero
void reset_servo_values(void)
	{
	ALB=0;
	ALJ1=0;
	ALJ2=0;
	ALC=0;
	ARB=0;
	ARJ1=0;
	ARJ2=0;
	ARC=0;
	CAMh=0;
	CAMv=0;
	}


//home position
//all positions are zeroed onto home position
//only needs to be called ONCE
void home_position(void)
	{
	reset_servo_values();
	robot_move();
	}


//gait interpolation + gait meshing, combined
//servo_position = (servo_gaitA[i]*perA+servo_gaitA[next]*perB)*percentage_gaitA
void interpolate_forward(float gait_per)
	{
	ALB=ALB + (ALB_forward[i]*per_a + ALB_forward[next]*per_b)*gait_per;
	ALJ1=ALJ1 + (ALJ1_forward[i]*per_a + ALJ1_forward[next]*per_b)*gait_per;
	ALJ2=ALJ2 + (ALJ2_forward[i]*per_a + ALJ2_forward[next]*per_b)*gait_per;
	ALC=ALC + (ALC_forward[i]*per_a + ALC_forward[next]*per_b)*gait_per;
	ARB=ARB + (ARB_forward[i]*per_a + ARB_forward[next]*per_b)*gait_per;
	ARJ1=ARJ1 + (ARJ1_forward[i]*per_a + ARJ1_forward[next]*per_b)*gait_per;
	ARJ2=ARJ2 + (ARJ2_forward[i]*per_a + ARJ2_forward[next]*per_b)*gait_per;
	ARC=ARC + (ARC_forward[i]*per_a + ARC_forward[next]*per_b)*gait_per;
	CAMh=CAMh + (CAMh_forward[i]*per_a + CAMh_forward[next]*per_b)*gait_per;
	CAMv=CAMv + (CAMv_forward[i]*per_a + CAMv_forward[next]*per_b)*gait_per;
	}
void interpolate_sidestep(float gait_per)
	{
	ALB=ALB + (ALB_sidestep[i]*per_a + ALB_sidestep[next]*per_b)*gait_per;
	ALJ1=ALJ1 + (ALJ1_sidestep[i]*per_a + ALJ1_sidestep[next]*per_b)*gait_per;
	ALJ2=ALJ2 + (ALJ2_sidestep[i]*per_a + ALJ2_sidestep[next]*per_b)*gait_per;
	ALC=ALC + (ALC_sidestep[i]*per_a + ALC_sidestep[next]*per_b)*gait_per;
	ARB=ARB + (ARB_sidestep[i]*per_a + ARB_sidestep[next]*per_b)*gait_per;
	ARJ1=ARJ1 + (ARJ1_sidestep[i]*per_a + ARJ1_sidestep[next]*per_b)*gait_per;
	ARJ2=ARJ2 + (ARJ2_sidestep[i]*per_a + ARJ2_sidestep[next]*per_b)*gait_per;
	ARC=ARC + (ARC_sidestep[i]*per_a + ARC_sidestep[next]*per_b)*gait_per;
	CAMh=CAMh + (CAMh_sidestep[i]*per_a + CAMh_sidestep[next]*per_b)*gait_per;
	CAMv=CAMv + (CAMv_sidestep[i]*per_a + CAMv_sidestep[next]*per_b)*gait_per;
	}
void interpolate_rotate(float gait_per)
	{
	ALB=ALB + (ALB_rotate[i]*per_a + ALB_rotate[next]*per_b)*gait_per;
	ALJ1=ALJ1 + (ALJ1_rotate[i]*per_a + ALJ1_rotate[next]*per_b)*gait_per;
	ALJ2=ALJ2 + (ALJ2_rotate[i]*per_a + ALJ2_rotate[next]*per_b)*gait_per;
	ALC=ALC + (ALC_rotate[i]*per_a + ALC_rotate[next]*per_b)*gait_per;
	ARB=ARB + (ARB_rotate[i]*per_a + ARB_rotate[next]*per_b)*gait_per;
	ARJ1=ARJ1 + (ARJ1_rotate[i]*per_a + ARJ1_rotate[next]*per_b)*gait_per;
	ARJ2=ARJ2 + (ARJ2_rotate[i]*per_a + ARJ2_rotate[next]*per_b)*gait_per;
	ARC=ARC + (ARC_rotate[i]*per_a + ARC_rotate[next]*per_b)*gait_per;
	CAMh=CAMh + (CAMh_rotate[i]*per_a + CAMh_rotate[next]*per_b)*gait_per;
	CAMv=CAMv + (CAMv_rotate[i]*per_a + CAMv_rotate[next]*per_b)*gait_per;
	}
void interpolate_stand(float gait_per)
	{
	ALB=ALB + (ALB_stand[i]*per_a + ALB_stand[next]*per_b)*gait_per;
	ALJ1=ALJ1 + (ALJ1_stand[i]*per_a + ALJ1_stand[next]*per_b)*gait_per;
	ALJ2=ALJ2 + (ALJ2_stand[i]*per_a + ALJ2_stand[next]*per_b)*gait_per;
	ALC=ALC + (ALC_stand[i]*per_a + ALC_stand[next]*per_b)*gait_per;
	ARB=ARB + (ARB_stand[i]*per_a + ARB_stand[next]*per_b)*gait_per;
	ARJ1=ARJ1 + (ARJ1_stand[i]*per_a + ARJ1_stand[next]*per_b)*gait_per;
	ARJ2=ARJ2 + (ARJ2_stand[i]*per_a + ARJ2_stand[next]*per_b)*gait_per;
	ARC=ARC + (ARC_stand[i]*per_a + ARC_stand[next]*per_b)*gait_per;
	CAMh=CAMh + (CAMh_stand[i]*per_a + CAMh_stand[next]*per_b)*gait_per;
	CAMv=CAMv + (CAMv_stand[i]*per_a + CAMv_stand[next]*per_b)*gait_per;
	}


//theta (200 degrees for circle) is direction, ex 0 is straight, 50 is side-step left, 100 is reverse, -50 is side-step right
//turning is robot rotation, ex 0 is no turning, 100 is turn left max, -100 is turn right max
//speed: higher numbers mean slower robot
void biped_engine(int16_t theta, int16_t turning, uint16_t speed)
	{
	/*
	//get starting clock time, and initialize timer variables
	TICK_COUNT start = clockGetus();
	TICK_COUNT end;
	TICK_COUNT duration;
	*/

	//limit values to be within operational range (stupid mistakes not allowed)
	theta	= CLAMP(theta, -100,100);
	turning = CLAMP(turning, -100,100);
	speed	= CLAMP(speed, 0,65000);


	for (i=0;i<array_count;i++)
		{
		//to test a particular position, uncomment i and j below and set it to a particular value
		//i=3;
		//rprintf("%d\n",i);

		//account for speed
		//interpolate between i and i+1, speed number of times
		for (j=0;j<speed;j++)
			{
			//j=0;//for testing particular kinematics points

			calculate_per(speed, array_count);//calculate interpolative division
			reset_servo_values();//reset all servo values from previous calculation

			gait_per_a=1;
			interpolate_forward(gait_per_a);

			/*
			
			//account for theta, do WGC
			if(theta <50 && theta >-50)//going forward
				{
				gait_per_a=100-abs(theta)*2;
				interpolate_forward(gait_per_a);
				}
			else if(theta >50 && theta <-50)//going reverse
				{
				//interpolate_reverse(abs(theta)*2-100);
				gait_per_a=-(abs(theta)*2-100);
				interpolate_forward(gait_per_a);
				}
			if(theta > 0)//going left
				{
				gait_per_b=abs(theta)*2-100;
				interpolate_sidestep(gait_per_b);
				}
			else if(theta < 0)//going right
				{
				gait_per_b=-(100-abs(theta)*2);
				interpolate_sidestep(gait_per_b);
				}
			if(speed>=stop)//if not moving, just stand there, duh!
				interpolate_stand(100-gait_per_a);
			
*/
			robot_move();//command fin to move

			//do something every now and then, like collect/output sensor data
			k++;
			if (k == (array_count*speed)/output_res)//output will happen output_res times per fin stroke
				{
				/*
				//output clock
				end = clockGetus();
				duration = end - start;
				start = end;//restart clock
				rprintf("%ld ",duration);//output clock
				*/
/*
				update_sensors_all();

				distanceDump(max_sonar);
				gyroDump(razor_lpr);
				gyroDump(razor_yaw);
				accelerometerDump(razor_accel);
*/
				k=0;//reset counter
				}
			}
		}
	}
