

#define servo_count 10

//Special Positions (make as many as you want)
//each position in the array is for a different servo
//each array is a single position
int16_t position_fold[servo_count]={0,-108,-127,-46,0,-111,-127,-54,0,-75};
int16_t position_reach_sky[servo_count]={0,0,0,0,0,0,0,0,0,-127};
int16_t position_worship_down[servo_count]={-3,123,17,-40,0,121,0,-36,0,41};
int16_t position_worship_up[servo_count]={0,21,43,14,0,15,29,16,0,-51};
int16_t position_worm_one[servo_count]={114,127,80,0,-104,125,60,0,25,-127};
int16_t position_worm_two[servo_count]={114,48,-127,0,-104,52,-127,0,-25,-127};
int16_t position_diamond[servo_count]={-109,-34,-127,0,102,-38,-127,0,0,-125};
int16_t position_claw1[servo_count]={13,-34,-127,-92,-11,-38,-127,102,0,-125};
int16_t position_claw2[servo_count]={13,-34,-127,92,-11,-38,-127,-104,0,-125};
int16_t position_wai_prep[servo_count]={0,64,40,0,0,66,32,0,0,-75};
int16_t position_wai[servo_count]={-28,118,88,-68,26,122,91,-72,0,0};
int16_t position_grab[servo_count]={0,-108,-127,-46,0,66,34,62,-26,-112};
int16_t position_grab_high[servo_count]={0,-108,-127,-46,8,69,-125,127,-12,-10};
int16_t position_grab_low[servo_count]={0,-108,-127,-46,8,123,-98,127,-14,18};

int16_t position_grab_1[servo_count]={0,-108,-127,-46,12,76,-127,127,0,26};//bring arm down
int16_t position_grab_2[servo_count]={0,-108,-127,-46,44,76,-127,127,0,26};//move arm to center
int16_t position_grab_3[servo_count]={0,-108,-127,-46,48,106,-127,127,0,26};//lower arm down again
int16_t position_grab_4[servo_count]={0,-108,-127,-46,48,106,-127,58,0,26};//grab ball
int16_t position_grab_5[servo_count]={0,-108,-127,-46,20,80,-127,58,0,26};//move arm back
int16_t position_grab_6[servo_count]={0,-108,-127,-46,0,-24,-75,58,0,26};//bring arm up


//Calculatable Positions
int16_t position_current[servo_count]={0,0,0,0,0,0,0,0,0,0};//store original positions
int16_t position_transit[servo_count]={0,0,0,0,0,0,0,0,0,0};//store transitional positions


void get_servo_positions(void)
	{
	position_current[0]=act_getSpeed(&arm_left_base);
	position_current[1]=act_getSpeed(&arm_left_J1);
	position_current[2]=act_getSpeed(&arm_left_J2);
	position_current[3]=act_getSpeed(&arm_left_claw);
	position_current[4]=act_getSpeed(&arm_right_base);
	position_current[5]=act_getSpeed(&arm_right_J1);
	position_current[6]=act_getSpeed(&arm_right_J2);
	position_current[7]=act_getSpeed(&arm_right_claw);
	position_current[8]=act_getSpeed(&camera_hor);
	position_current[9]=act_getSpeed(&camera_ver);

/*
	//TEST output data
	rprintf("\n current: ");
	for(int i=0;i<servo_count;i++)
		rprintf("%d ",position_current[i]);
	//delay_ms(5000);
*/
	}

void calculate_transition(float percent, int16_t position_desired[servo_count])
	{
	int i;

	for(i=0;i<servo_count;i++)
		//new_position = current_position*(1-percent) + desired position*percent;
		position_transit[i]=position_current[i]*(float)(1.0-percent) + position_desired[i]*(float)percent;
		//position_transit[i]=position_current[i]/2 + position_desired[i]/2;

/*
	//TEST output data
	rprintf("\n transit: ");
	for(int i=0;i<servo_count;i++)
		rprintf("%d ",position_transit[i]);
*/
	}

void robot_move_special(void)
	{
	act_setSpeed(&arm_left_base, position_transit[0]);
	act_setSpeed(&arm_left_J1, position_transit[1]);
	act_setSpeed(&arm_left_J2, position_transit[2]);
	act_setSpeed(&arm_left_claw, position_transit[3]);
	act_setSpeed(&arm_right_base, position_transit[4]);
	act_setSpeed(&arm_right_J1, position_transit[5]);
	act_setSpeed(&arm_right_J2, position_transit[6]);
	act_setSpeed(&arm_right_claw, position_transit[7]);
	act_setSpeed(&camera_hor, position_transit[8]);
	act_setSpeed(&camera_ver, position_transit[9]);
	}

void servo_transistion(float speed, int16_t gait[servo_count])
	{
	float per;//percentage between each gait

	//find out where all servos are
	get_servo_positions();

	//calculation transition positions
	for(per=0.0;per<1.0;per=per+speed)
		{
		//TEST output percentage
		//rprintf("\n per:");
		//rprintfFloat(5,per);

		//determine new angles
		calculate_transition(per,gait);

		//move to new angles
		robot_move_special();

		//slow it down - transistions too fast
		delay_ms(10);
		}
	}

//make arm go into folding position
void arm_fold(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_fold);
	}

//make arm go into folding position
void arm_reach_sky(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_reach_sky);
	}

//make arm go into folding position
void arm_worm_one(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_worm_one);
	}

//make arm go into folding position
void arm_worm_two(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_worm_two);
	}

//make arm go into folding position
void arm_diamond(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_diamond);
	}

//make arm go into folding position
void arm_worship_down(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_worship_down);
	}

//make arm go into folding position
void arm_worship_up(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_worship_up);
	}

//make arm go into folding position
void arm_claw_attack1(void)
	{
	float transit_speed=0.1;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_claw1);
	}

//make arm go into folding position
void arm_claw_attack2(void)
	{
	float transit_speed=0.1;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_claw2);
	}

//make arm go into folding position
void arm_wai_prep(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_wai_prep);
	}

//make arm go into folding position
void arm_wai(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_wai);
	}

//make arm go into folding position
void arm_grab_high(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_high);
	}

//make arm go into folding position
void arm_grab_low(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_low);
	}


//pick up object commands
void arm_grab_1(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_1);
	}
//pick up object commands
void arm_grab_2(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_2);
	}
//pick up object commands
void arm_grab_3(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_3);
	}
//pick up object commands
void arm_grab_4(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_4);
	}
//pick up object commands
void arm_grab_5(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_5);
	}
//pick up object commands
void arm_grab_6(void)
	{
	float transit_speed=0.01;//make larger to go faster, max at 1.0

	servo_transistion(transit_speed,position_grab_6);
	}
void arm_grab_center(void)
	{
	arm_grab_1();
	delay_ms(200);
	arm_grab_2();
	delay_ms(200);
	arm_grab_3();
	delay_ms(200);
	arm_grab_4();
	delay_ms(200);
	arm_grab_5();
	delay_ms(200);
	arm_grab_6();
	}
void arm_put_down_object(void)
	{
	arm_grab_5();
	delay_ms(200);
	arm_grab_4();
	delay_ms(200);
	arm_grab_3();
	delay_ms(200);
	arm_grab_2();
	delay_ms(200);
	arm_grab_1();
	}

//wheel commands
void wheels_forward(void)
	{
	act_setSpeed(&wheel_left, DRIVE_SPEED_MAX);
	act_setSpeed(&wheel_right,DRIVE_SPEED_MAX);
	}
void wheels_reverse(void)
	{
	act_setSpeed(&wheel_left, DRIVE_SPEED_MIN);
	act_setSpeed(&wheel_right,DRIVE_SPEED_MIN);
	}
void wheels_left(void)
	{
	act_setSpeed(&wheel_left, DRIVE_SPEED_MIN);
	act_setSpeed(&wheel_right,DRIVE_SPEED_MAX);
	}
void wheels_right(void)
	{
	act_setSpeed(&wheel_left, DRIVE_SPEED_MAX);
	act_setSpeed(&wheel_right,DRIVE_SPEED_MIN);
	}
void wheels_stop(void)
	{
	act_setSpeed(&wheel_left, 0);
	act_setSpeed(&wheel_right,0);
	}
void head_straight(void)
	{
	act_setSpeed(&camera_hor,0);
	act_setSpeed(&camera_ver,-75);
	}
